#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include "rgma.h"

static long getCurrentTime(void);
static long getTupleAge(const char *measurementDate, const char *measurementTime, long now);
static long sec_since_1970(int year, int mon, int mday, int hour, int min, int sec);

int main(int argc, char *argv[])
{ 
    RGMAResource *r;
    RGMAResourceEndpoint *endpoint;
    RGMAResultSet *rs;
    char *testId, *tableName, *queryTypeS, *select,
         *measurementDate, *measurementTime;
    int terminationInterval, queryTimeout, numTuples, queryType, oldness,
        startTime, currentTime, tupleAge, maxTupleAge, i, endOfResults;

    /* Check arguments. */

    if (argc != 5)
    {
	fprintf(stderr, "Usage: <terminationInterval> <queryTimeout> <queryType> <select>\n");
        exit(1);
    }

    terminationInterval = atoi(argv[1]); //seconds
    queryTimeout = atoi(argv[2]); //seconds
    queryTypeS = argv[3];
    select = argv[4];

    /* Create Consumer to consume from tableName. */

    startTime = time(NULL);

    if (strcmp(queryTypeS, "CONTINUOUS") == 0)
    {
        oldness = queryTimeout;
        queryType = RGMAQueryType_CONTINUOUS | RGMAQueryType_INTERVAL;
    }
    else if (strcmp(queryTypeS, "LATEST") == 0)
    {
        oldness = 0;
        queryType = RGMAQueryType_LATEST;
    }
    else if (strcmp(queryTypeS, "HISTORY") == 0)
    {
        oldness = 0;
        queryType = RGMAQueryType_HISTORY;
    }
    else
    {
        fprintf(stderr, "Invalid query type (%s) in test script.\n");
        exit(1);
    }

    if (RGMA_createConsumer(terminationInterval, select, queryType, oldness, &r) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    endpoint = RGMA_getEndpoint(r);
    if (endpoint == NULL)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    if (RGMAConsumer_start(r, queryTimeout) != 0)
    {
        fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
        exit(1);
    }

    /* Wait for tuples: since we're timing things, we ought to have a short
       timeout in the sleep statement. */

    numTuples = 0;
    endOfResults = 0;
    maxTupleAge = 0;
    currentTime = getCurrentTime();
    if (strcmp(queryTypeS, "CONTINUOUS") == 0) sleep(queryTimeout);
    while (!endOfResults)
    {
        endOfResults = (!RGMAConsumer_isExecuting(r));

        rs = RGMAConsumer_pop(r, 1000); // get chunks of up to 1000 tuples
        if (rs == NULL)
        {
            fprintf(stderr, "%s\n", RGMA_getException(r)->errorMessage);
            exit(1);
        }
        numTuples += rs->numRows;
        for (i = 0; i < rs->numRows; ++i)
        {
            measurementDate = RGMA_getResultSetValue(rs, i, 0, "MeasurementDate");
            measurementTime = RGMA_getResultSetValue(rs, i, 0, "MeasurementTime");
            tupleAge = getTupleAge(measurementDate, measurementTime, currentTime);
    	    if (tupleAge > maxTupleAge) maxTupleAge = tupleAge;
        }
        RGMA_freeResultSet(rs);

        sleep(1);
    }
    
    printf("URL=%s\n", endpoint->url);
    printf("resourceId=%d\n", endpoint->resourceId);
    printf("startTime=%d\n", startTime);
    printf("endTime=%d\n", time(NULL));
    printf("numTuples=%d\n", numTuples);
    printf("maxTupleAge=%d\n", maxTupleAge);

    /* Clean up. */

    RGMA_close(r);
    RGMA_freeEndpoint(endpoint);
    RGMA_freeResource(r);
    exit(0);
}

/* Return the current time in UTC as an offset from 1970. We use our own
   calculation for the offset (rather than just using the value of time()
   itself) because there's no way in the standard library of converting an
   arbitrary UTC time to the same format for comparison (mktime() only works
   for local time). J.A.W. */

static long getCurrentTime(void)
{
    time_t now;
    struct tm *now_utc;

    now = time(NULL);
    now_utc = gmtime(&now);
    if (now_utc == NULL) return -1;

    return (sec_since_1970(now_utc->tm_year, now_utc->tm_mon,
                           now_utc->tm_mday, now_utc->tm_hour,
                           now_utc->tm_min, now_utc->tm_sec));
}

/* Return the offset of the UTC tuple time from "now". Returns 0 on error
   as that's not going to hugely mess up the tests. */

static long getTupleAge(const char *measurementDate, const char *measurementTime, long now)
{
    int sec, min, hour, mday, mon, year;
    long tuple_time;

    if (measurementDate == NULL || measurementTime == NULL || now == -1)
    {
        return 0;
    }

    if (sscanf(measurementDate, "%d-%d-%d", &year, &mon, &mday) != 3 ||
        sscanf(measurementTime, "%d:%d:%d", &hour, &min, &sec) != 3)
    {
        return 0;
    }

    tuple_time = sec_since_1970(year - 1900, mon - 1, mday, hour, min, sec);
    if (tuple_time == -1)  return 0;

    return (now - tuple_time);
}

/* Cheap and cheerful function to calculate offset in seconds since the
   epoch (00:00:00 UTC, January 1, 1970). The parameters here correspond to
   the defintions of the fields in "struct tm" in <time.h>. This is only for
   comparing tuple times for testing purposes (which will be recent and
   in the same or consecutive years), so this code is good enough. */

static long sec_since_1970(int year, int mon, int mday, int hour, int min, int sec)
{
    static int DAYS[] = {31,28,31,30,31,30,31,31,30,31,30,31};
    long days, y, m;

    if (year < 70 || mon < 0 || mon > 11 || mday < 1 || mday > 31 ||
        hour < 0 || hour > 23 || min < 0 || min > 59 || sec < 0 || sec > 61)
    {
        return -1;
    }

    year += 1900;  /* make it an absolute year */
    mday -= 1;     /* make it in range 0 - 30 */

    days = 0;

    for (y = 1970; y < year; ++y)
    {
        if (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0)) days += 366;
        else days += 365;
    }

    for (m = 0; m < mon; ++m)
    {
        days += DAYS[m];
    }

    if (mon > 1) /* month 1 is February */
    {
        if (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0)) ++days;
    }
        
    days += mday;

    return (days * 24 * 60 * 60 + hour * 60 * 60 + min * 60 + sec);
}

/* End of file. */
