#include "impl/ConsumerFactoryImpl.h"

#include <string>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

using rgma::impl::ConsumerFactoryImpl;
using glite::rgma::ConsumerFactory;
using glite::rgma::Consumer;
using glite::rgma::QueryProperties;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::ResultSet;
using glite::rgma::ResourceEndpoint;
using glite::rgma::ExceptionBase;

static long getCurrentTime(void);
static long getTupleAge(const char *measurementDate, const char *measurementTime, long now);
static long sec_since_1970(int year, int mon, int mday, int hour, int min, int sec);

int main(int argc, char *argv[])
{ 
    /* Check arguments. */

    if (argc != 5)
    {
        std::cerr << "Usage: <terminationInterval> <queryTimeout> <queryType> <select>\n" << std::endl;
        exit(1);
    }
    int terminationInterval = atoi(argv[1]); //seconds
    int queryTimeout = atoi(argv[2]); //seconds
    std::string queryType(argv[3]);
    std::string select(argv[4]);

    /* Create Consumer to consume from tableName. Do a continuous+old
       query in case the producer manages to start up more quickly than
       the consumer. */

    int startTime = time(NULL);
    ResourceEndpoint endpoint;

    ConsumerFactory *cfactory = new ConsumerFactoryImpl();
    Consumer *c;

    try
    {
        TimeInterval terminationInterval(terminationInterval, Units::SECONDS);
        if (queryType == "CONTINUOUS")
        {
            TimeInterval oldness(queryTimeout, Units::SECONDS);
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::getContinuous(oldness));
        }
        else if (queryType == "LATEST")
        {
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::LATEST);
        }
        else if (queryType == "HISTORY")
        {
            c = cfactory->createConsumer(terminationInterval, 
                                         select,
                                         QueryProperties::HISTORY);
        }
        else
        {
            std::cerr << "Invalid query type (" << queryType << ") in test script.\n";
            exit(1);
        }

        endpoint = c->getEndpoint();
        c->start(TimeInterval(queryTimeout, Units::SECONDS));
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to create/start consumer."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    /* Wait for tuples: since we're timing things, we ought to have a short
       timeout in the sleep statement. */

    int numTuples = 0;
    long currentTime = getCurrentTime();
    long maxTupleAge = 0;
    
    try
    {
	int endOfResults = 0;
        if (queryType == "CONTINUOUS") sleep(queryTimeout);
        while(!endOfResults)
        {
            endOfResults = (!c->isExecuting());
    
            ResultSet resultSet;
            c->pop(resultSet, 1000); // get chunks of 1000 tuples
            ResultSet::iterator i;
            long tupleAge = 0;
            std::string measurementDate;
            std::string measurementTime;
            for (i = resultSet.begin(); i < resultSet.end(); ++i)
            {
                ++numTuples;
                measurementDate = i->getString("MeasurementDate");
                measurementTime = i->getString("MeasurementTime");
                tupleAge = getTupleAge(measurementDate.c_str(),
                                       measurementTime.c_str(),
                                       currentTime);
                if (tupleAge > maxTupleAge) maxTupleAge = tupleAge;
            }

            sleep(1);
        }
    }
    catch (ExceptionBase e)
    {
        std::cerr << "Failed to retrieve results."<< std::endl;
        std::cerr << e.getMessage() << std::endl;
        exit(1);
    }

    std::cout << "URL=" << endpoint.getUrl() << "\n";
    std::cout << "resourceId=" << endpoint.getResourceId() << "\n";
    std::cout << "startTime=" << startTime << "\n";
    std::cout << "endTime=" << time(NULL) << "\n";
    std::cout << "numTuples=" << numTuples << "\n";
    std::cout << "maxTupleAge=" << maxTupleAge << "\n";

    /* Clean up. */

    c->close();
    delete cfactory;
    exit(0);
}

/* NB. The remainder of this code is identical to test_select.c and needs to
   remain that way! */

/* Return the current time in UTC as an offset from 1970. We use our own
   calculation for the offset (rather than just using the value of time()
   itself) because there's no way in the standard library of converting an
   arbitrary UTC time to the same format for comparison (mktime() only works
   for local time). J.A.W. */

static long getCurrentTime(void)
{
    time_t now;
    struct tm *now_utc;

    now = time(NULL);
    now_utc = gmtime(&now);
    if (now_utc == NULL) return -1;

    return (sec_since_1970(now_utc->tm_year, now_utc->tm_mon,
                           now_utc->tm_mday, now_utc->tm_hour,
                           now_utc->tm_min, now_utc->tm_sec));
}

/* Return the offset of the UTC tuple time from "now". Returns 0 on error
   as that's not going to hugely mess up the tests. */

static long getTupleAge(const char *measurementDate, const char *measurementTime, long now)
{
    int sec, min, hour, mday, mon, year;
    long tuple_time;

    if (measurementDate == NULL || measurementTime == NULL || now == -1)
    {
        return 0;
    }

    if (sscanf(measurementDate, "%d-%d-%d", &year, &mon, &mday) != 3 ||
        sscanf(measurementTime, "%d:%d:%d", &hour, &min, &sec) != 3)
    {
        return 0;
    }

    tuple_time = sec_since_1970(year - 1900, mon - 1, mday, hour, min, sec);
    if (tuple_time == -1)  return 0;

    return (now - tuple_time);
}

/* Cheap and cheerful function to calculate offset in seconds since the
   epoch (00:00:00 UTC, January 1, 1970). The parameters here correspond to
   the defintions of the fields in "struct tm" in <time.h>. This is only for
   comparing tuple times for testing purposes (which will be recent and
   in the same or consecutive years), so this code is good enough. */

static long sec_since_1970(int year, int mon, int mday, int hour, int min, int sec)
{
    static int DAYS[] = {31,28,31,30,31,30,31,31,30,31,30,31};
    long days, y, m;

    if (year < 70 || mon < 0 || mon > 11 || mday < 1 || mday > 31 ||
        hour < 0 || hour > 23 || min < 0 || min > 59 || sec < 0 || sec > 61)
    {
        return -1;
    }

    year += 1900;  /* make it an absolute year */
    mday -= 1;     /* make it in range 0 - 30 */

    days = 0;

    for (y = 1970; y < year; ++y)
    {
        if (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0)) days += 366;
        else days += 365;
    }

    for (m = 0; m < mon; ++m)
    {
        days += DAYS[m];
    }

    if (mon > 1) /* month 1 is February */
    {
        if (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0)) ++days;
    }
        
    days += mday;

    return (days * 24 * 60 * 60 + hour * 60 * 60 + min * 60 + sec);
}

/* End of file. */
