# Test initialisation script - sets up test environment.

# Test scripts should dot this file and pass the test source directory
# (absolute path) and the test number in as arguments.

# Redirect all uncaught standard output to standard error.

exec 1>&2

# Define test globals (shared by all test scripts).

TEST_SRCDIR=$1
TEST_NUMBER=$2

TEST_ID="TEST$TEST_NUMBER.$$"
TEST_DIR=`pwd`/test${TEST_NUMBER}_$$ # Working dir.

TEST_SUCCESS=0      # Must match the codes in checkstatus.cpp
TEST_FAILURE=1      # Must match the codes in checkstatus.cpp
TEST_WAIT=2         # Must match the codes in checkstatus.cpp
TEST_ABANDON=3      # Must match the codes in checkstatus.cpp
TEST_FAILURE_MESSAGE="Test $TEST_NUMBER FAILED"
TEST_SUCCESS_MESSAGE="Test $TEST_NUMBER SUCCEEDED"

TEST_PROPFILE=$TEST_SRCDIR/testprops.txt
TEST_JDL_TEMPLATE=$TEST_SRCDIR/lib/test_template.jdl
TEST_C_JOB_TEMPLATE=$TEST_SRCDIR/lib/test_c_template.job
TEST_PP_JOB_TEMPLATE=$TEST_SRCDIR/lib/test_pp_template.job

TEST_SRC_CURRENTTIME=$TEST_SRCDIR/lib/currenttime.c
TEST_SRC_CHECKSTATUS=$TEST_SRCDIR/lib/checkstatus.c

TEST_AWK=awk
TEST_BASENAME=basename
TEST_CAT=cat
TEST_CC=gcc         # for C API tests and utilities
TEST_CP=cp
TEST_CPP=g++        # for C++ API tests only
TEST_EXPR=expr
TEST_JAVAC=javac    # for Java API tests only
TEST_MKDIR=mkdir
TEST_PYTHON=python2 # for Python API tests only
TEST_RM=rm
TEST_SED=sed
TEST_SLEEP=sleep
TEST_JOBSUBMIT=glite-job-submit
TEST_JOBSTATUS=glite-job-status
TEST_JOBOUTPUT=glite-job-output

# Load common function definitions (shared by this script and the remote job
# scripts).

TEST_FUNCTIONS=$TEST_SRCDIR/lib/test_functions.sh
. $TEST_FUNCTIONS # currently just defines TEST_JAVA

# Function to get read value from property file (this is used to read
# configuration properties from TEST_PROPFILE and to read results from the
# API programs).

function TEST_GETPROPERTY()
{
        local file=$1
        local key=$2
                                                                                
        local value=`$TEST_AWK -F# "{print \\$1}" $file |
                     $TEST_AWK -F= "/^$key/ {print \\$2; exit}"`

        if [ "$value" = "" ]
        then
                echo "Key $key not found in file $file."
                echo $TEST_FAILURE_MESSAGE
                exit $TEST_FAILURE
        fi

	echo $value
}

# Function to compile utilities (these are all written in C and compiled on
# the fly to avoid portability problems with the executable code).

function TEST_COMPILE_UTIL()
{
	local test_src=$1
	local test_exe=$2

	$TEST_CC $test_src -o $test_exe -O

	if [ $? -ne 0 -o ! -x $test_exe ]
	then
	        echo "Compilation of $test_src failed."
	        echo $TEST_FAILURE_MESSAGE
	        exit $TEST_FAILURE
	fi
}

# Get API-specific configuration parameters (TEST_COMPILE, TEST_RUN and the
# TEST_SRC_* file names).

TEST_API=`TEST_GETPROPERTY $TEST_PROPFILE TEST_API`

if [ "$TEST_API" = "CPP" ]
then
	. $TEST_SRCDIR/lib/testinit_cpp.sh
elif [ "$TEST_API" = "JAVA" ]
then
	. $TEST_SRCDIR/lib/testinit_java.sh
elif [ "$TEST_API" = "C" ]
then
	. $TEST_SRCDIR/lib/testinit_c.sh
elif [ "$TEST_API" = "PYTHON" ]
then
	. $TEST_SRCDIR/lib/testinit_py.sh
else
	echo "Only the CPP API is currently supported."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Get the remaining user-configurable properties.
	
TEST_tableName=`TEST_GETPROPERTY $TEST_PROPFILE TEST_tableName`
TEST_terminationInterval=`TEST_GETPROPERTY $TEST_PROPFILE TEST_terminationInterval`
TEST_historyRP=`TEST_GETPROPERTY $TEST_PROPFILE TEST_historyRP`
TEST_latestRP=`TEST_GETPROPERTY $TEST_PROPFILE TEST_latestRP`
TEST_numTuples=`TEST_GETPROPERTY $TEST_PROPFILE TEST_numTuples`
TEST_queryTimeout=`TEST_GETPROPERTY $TEST_PROPFILE TEST_queryTimeout`
TEST_tupleFactor=`TEST_GETPROPERTY $TEST_PROPFILE TEST_tupleFactor`
test $TEST_tupleFactor -ge 1 || TEST_tupleFactor=$TEST_numTuples
TEST_jobWaitTime=`TEST_GETPROPERTY $TEST_PROPFILE TEST_jobWaitTime`
TEST_jobPollTime=`TEST_GETPROPERTY $TEST_PROPFILE TEST_jobPollTime`
TEST_numJobs=`TEST_GETPROPERTY $TEST_PROPFILE TEST_numJobs`

# Check R-GMA Client environment is properly set up.

if [ -z "$GLITE_LOCATION" ]
then
	echo "GLITE_LOCATION variable is not set."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

defaultProxyFile=/tmp/x509up_u${UID}
if [ -z "$X509_USER_PROXY" -a -f $defaultProxyFile ]
then
	export X509_USER_PROXY=$defaultProxyFile
fi

if [ -z "$X509_USER_PROXY" -a -z "$TRUSTFILE" ]
then
	echo "Client security is not properly configured."
	echo "Either X509_USER_PROXY or TRUSTFILE must be set."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

if [ -z "$RGMA_HOME" ]
then
	export RGMA_HOME=$GLITE_LOCATION
fi

if [ -z "$LD_LIBRARY_PATH" ]
then
	export LD_LIBRARY_PATH=$RGMA_HOME/lib:$GLITE_LOCATION/externals/lib
else
	export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$RGMA_HOME/lib:$GLITE_LOCATION/externals/lib
fi

if [ -z "$PYTHONPATH" ]
then
	export PYTHONPATH=$RGMA_HOME/lib/python
else
	export PYTHONPATH=$PYTHONPATH:$RGMA_HOME/lib/python
fi

# Create the test directory.

$TEST_RM -rf $TEST_DIR
$TEST_MKDIR $TEST_DIR
if [ $? -ne 0 ]
then
	echo "Can't create test environment (can't create $TEST_DIR)."
	echo $TEST_FAILURE_MESSAGE
	exit $TEST_FAILURE
fi

# Log details of the test environment.

echo "Running test $TEST_NUMBER ($TEST_ID) in $TEST_DIR with:"
echo "  GLITE_LOCATION: $GLITE_LOCATION"
echo "  RGMA_HOME: $RGMA_HOME"
echo "  LD_LIBRARY_PATH: $LD_LIBRARY_PATH"
echo "  TRUSTFILE: $TRUSTFILE"
echo "  X509_USER_PROXY: $X509_USER_PROXY"

# End of file.
